<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <title>DVH Database</title>
    {{ bokeh_css }}
    {{ bokeh_js }}
  </head>
  <body>
    <div class="content">
      <p style="font-family:helvetica;"><img src="/dvh/static/images/logo.jpg" style="height:40px;"></p>
      {{ plot_div|indent(8) }}
        {{ plot_script|indent(8) }}
    </div>
  </body>
</html>